#!/usr/bin/env python
"""sorno_rename.py renames files given regex for matching names of the
existing files and using backreferences for filenames to be renamed to.

Backreferences are denoted by {0}, {1}, {2}...{0} is the entire match, and {1}
is the first matching group, etc.

The script shows you all the files to be renamed and what they will be renamed
to and prompts for a confirmation before executing the renaming.

    Copyright 2015 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
import logging
import os
import re
import sys

from sorno import (
    consoleutil,
    loggingutil,
    stringutil,
)


_log = logging.getLogger()
_plain_logger = None  # will be created in main()
_plain_error_logger = None  # will be created in main()


class App(object):
    """A console application to do work"""
    def __init__(
        self,
        args,
    ):
        """
        Args:
            args (argparse.Namespace): The flags for the script.
        """
        self.args = args
        self.filenames_map = {}

    def run(self):
        r = stringutil.u(self.args.regex)
        s = stringutil.u(self.args.sub)
        if self.args.recursive:
            for dirpath, dirs, filenames in os.walk(self.args.dir):
                for fname in filenames:
                    self.process_file(r, s, dirpath, fname)
        else:
            filenames = os.listdir(self.args.dir)
            for fname in filenames:
                self.process_file(r, s, self.args.dir, fname)

        n = len(self.filenames_map)
        sn = len(str(n))

        for i, (k, v) in enumerate(self.filenames_map.iteritems()):
            print(u"%*d. %s => %s" % (sn, i + 1, k, v))

        if consoleutil.confirm("Do you want to do the rename?"):
            for i, (k, v) in enumerate(self.filenames_map.iteritems()):
                print("%*d. Rename %s to %s" % (sn, i + 1, k, v))
                os.rename(k, v)
            return 0
        else:
            print("Aborted")
            return 1

    def process_file(self, regex, sub, dirpath, filename):
        filename = stringutil.u(filename)
        dirpath = stringutil.u(dirpath)
        m = re.match(regex, filename, re.UNICODE)
        if m:
            old = os.path.abspath(os.path.join(dirpath, filename))
            groups = (m.group(0),) + m.groups()
            new = os.path.abspath(
                os.path.join(
                    dirpath,
                    sub.format(*groups),
                )
            )
            self.filenames_map[old] = new


def parse_args(cmd_args):
    description = __doc__.split("Copyright 2015")[0].strip()

    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )
    parser.add_argument(
        "dir",
        nargs="?",
        help=stringutil.oneline(
            """
            The directory that contains the files (default "%(default)s"). The
            script traverses all the files in the directory and
            subdirectories.
            """
        ),
        default=".",
    )
    parser.add_argument(
        "-R",
        "--recursive",
        help=stringutil.oneline(
            """
            Recursively goes to subdirectories to look for files to be renamed
            """
        ),
        action="store_true",
    )
    parser.add_argument(
        "--regex",
        help="Regex to match filenames",
        default='.*',
    )
    parser.add_argument(
        "--sub",
        help=stringutil.oneline(
            """
            The new filenames. You can use backreferences which are denoted by
            {0}, {1}, {2}...{0} is the entire match, and {1} is the first"
            matching group, etc.
            """
        ),
        default="{0}",
    )

    args = parser.parse_args(cmd_args)
    return args


def main():
    global _plain_logger, _plain_error_logger

    args = parse_args(sys.argv[1:])

    loggingutil.setup_logger(_log, debug=args.debug)
    _plain_logger = loggingutil.create_plain_logger(
        "PLAIN",
        debug=args.debug,
    )
    _plain_error_logger = loggingutil.create_plain_logger(
        "PLAIN_ERROR",
        debug=args.debug,
        stdout=False,
    )

    app = App(
        args,
    )
    sys.exit(app.run())


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""Fills up the disk space with a specific size of garbage data.


   Copyright 2014 Heung Ming Tai

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
import logging
import os
import sys


_LOG = logging.getLogger(__name__)


class SpaceFiller(object):
    def __init__(
        self,
        size,
        output_file,
        buffer_size,
    ):
        """
        @param size
            In GB.
        @param output_file
        @param buffer_size
        """

        self.size = size
        self.output_file = output_file
        self.buffer_size = buffer_size

    def run(self):
        # bytes to fillup
        num_remaining_bytes = int(1024 * 1024 * 1024 * self.size)
        with open(self.output_file, "wb") as f:
            while num_remaining_bytes > 0:
                # do it 10 MB by 10 MB
                _LOG.info(
                    "%d MB remaining.",
                    num_remaining_bytes / (1024 * 1024),
                )
                num_bytes_to_write = min(
                    num_remaining_bytes,
                    self.buffer_size * 1024 * 1024,
                )
                bytes_to_write = os.urandom(num_bytes_to_write)
                f.write(bytes_to_write)
                num_remaining_bytes -= num_bytes_to_write

        _LOG.info("Done")


#
# logger-related functions
#

def setup_logger(
    logger,
    debug=False,
    stdout=False,
    add_thread_id=False,
    logging_level=None,
    use_path=False,
):
    if logging_level is None:
        if debug:
            logging_level = logging.DEBUG
        else:
            logging_level = logging.INFO

    formatter = create_logging_formatter(
        add_thread_id=add_thread_id,
        use_path=use_path
    )
    hdlr = create_stream_handler(
        formatter=formatter,
        stdout=stdout,
    )
    logger.handlers = []  # clear the existing handlers
    logger.addHandler(hdlr)
    logger.setLevel(logging_level)


def create_logging_formatter(add_thread_id=False, use_path=False):
    format_str = "%(asctime)s"

    if add_thread_id:
        format_str += " thread:%(thread)s"

    format_str += " %(levelname)s "

    if use_path:
        format_str += "%(pathname)s"
    else:
        format_str += "%(name)s"

    format_str += ":%(lineno)s: %(message)s"

    detail_formatter = logging.Formatter(
        fmt=format_str,
        datefmt="%Y-%m-%d %H:%M:%S"
    )
    return detail_formatter


def create_stream_handler(formatter=None, stdout=False):
    if formatter is None:
        formatter = create_logging_formatter()

    if stdout:
        stream = sys.stdout
    else:
        stream = sys.stderr

    hdlr = logging.StreamHandler(stream=stream)
    hdlr.setFormatter(formatter)
    return hdlr

#
# end of logger-related functions
#

def parse_args(cmd_args):
    description = """
Fill up the disk space with a specific size of garbage data.
    """
    parser = argparse.ArgumentParser(
        description=description,
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )
    parser.add_argument(
        "--buffer-size",
        type=int,
        default=10,
        help="Buffer size in MB",
    )
    parser.add_argument(
        "--size",
        required=True,
        type=float,
        help="In GB",
    )
    parser.add_argument("--output", default="stub.dat")

    args = parser.parse_args(cmd_args)
    return args


def main():
    args = parse_args(sys.argv[1:])

    setup_logger(_LOG, debug=args.debug)

    app = SpaceFiller(
        args.size,
        args.output,
        args.buffer_size,
    )
    app.run()


if __name__ == '__main__':
    main()



"""Utilities for web related stuff


Copyright 2014 Heung Ming Tai

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import requests
import urllib


def unquote_url(url):
    return urllib.unquote(url).decode('utf8')


def download_file(url, local_filename):
    r = requests.get(url, stream=True)
    with open(local_filename, 'wb') as f:
        for chunk in r.iter_content(chunk_size=1024 * 1024):
            if chunk: # filter out keep-alive new chunks
                f.write(chunk)
                f.flush()



"""Utility classes and functions which have random purpose
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals


class Bunch:
    """A class that allows to add fields at construction time and at runtime
    http://code.activestate.com/recipes/52308-the-simple-but-handy-collector-of-a-bunch-of-named/?in=user-97991
    """
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)

    def __str__(self):
        return str(self.__dict__)

    def __repr__(self):
        return repr(self.__dict__)



"""
Utility functions for dealing with strings
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import re


def oneline(s):
    """
    Compact all space characters to a single space. Leading and trailing
    spaces are stripped.
    """
    return re.sub(r"\s+", " ", s).strip()


def u(s):
    """
    Converts s to unicode with utf-8 encoding if it is not already a unicode.
    Leave it as is otherwise.
    """
    if type(s) == unicode:
        return s
    else:
        return s.decode("utf8")


def format_with_default_value(handle_missing_key, s, d):
    """Formats a string with handling of missing keys from the dict.

    Calls s.format(**d) while handling missing keys by calling
    handle_missing_key to get the appropriate values for the missing keys.

    Args:
        handle_issing_key: A function that takes a missing key as the argument
            and returns a value for the value of the missing key.
        s: A format string.
        d: A dict providing values to format s.

    Returns s.format(**d) with missing keys handled by calling
    handle_missing_key to get the values for the missing keys.
    """
    copy = dict(**d)
    while True:
        try:
            return s.format(**copy)
        except KeyError as ex:
            key = ex.args[0]
            copy[key] = handle_missing_key(key)



#!/usr/bin/env python
"""sorno_ls.py is just like the Unix "ls" command


    Copyright 2016 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import argparse
import glob
import sys


class App(object):
    def __init__(self, args):
        self.args = args

    def run(self):
        if not self.args.files:
            patterns = ["*"]
        else:
            patterns = self.args.files

        for p in patterns:
            print("\n".join(glob.glob(p)))
        return 0


def parse_args(cmd_args):
    description = __doc__.split("Copyright 2016")[0].strip()

    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument("files", nargs="*", metavar="file")

    args = parser.parse_args(cmd_args)
    return args


def main():
    args = parse_args(sys.argv[1:])

    app = App(args)
    sys.exit(app.run())


if __name__ == '__main__':
    main()



#!/usr/bin/python
"""A command line client for Google Drive.

The API doc used to implement this is in
https://developers.google.com/drive/web/quickstart/quickstart-python

Currently, you can upload files with the script.

In order to use this script, please look at the "Using scripts involve Google
App API" section of the sorno-py-scripts README (can be found in
https://github.com/hermantai/sorno-scripts/tree/master/sorno-py-scripts). The
API needed for this script is "Drive API" with the scope
'https://www.googleapis.com/auth/drive.readonly'

Examples:

    You can upload files by running:

        $ sorno_gdrive.py upload <file1> [file2, file3...]


    Copyright 2014 Heung Ming Tai

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

import argparse
import logging
import os
import pprint
import subprocess
import sys
import tempfile

import apiclient
from apiclient.discovery import build
from apiclient.http import MediaFileUpload

import httplib2

from oauth2client.client import OAuth2WebServerFlow
from oauth2client.file import Storage

from sorno import consoleutil
from sorno import loggingutil
from sorno import webutil

_LOG = logging.getLogger(__name__)

# Check https://developers.google.com/drive/scopes for all available scopes
OAUTH_SCOPE = 'https://www.googleapis.com/auth/drive'

# Redirect URI for installed apps
REDIRECT_URI = 'urn:ietf:wg:oauth:2.0:oob'

CREDENTIALS_FILE = os.path.expanduser("~/.sorno_gdrive-google-drive-api.cred")

GDRIVE_MIMETYPE_FOLDER = "application/vnd.google-apps.folder"


class App(object):
    def __init__(self):
        self.drive_service = None

    def auth(self, use_credentials_cache=True):
        # Copy your credentials from the console
        client_id = os.getenv('GOOGLE_APP_PROJECT_CLIENT_ID')
        client_secret = os.getenv('GOOGLE_APP_PROJECT_CLIENT_SECRET')

        if not client_id:
            print(
                "Please set the environment variable"
                " GOOGLE_APP_PROJECT_CLIENT_ID"
            )
            sys.exit(1)

        if not client_secret:
            print(
                "Please set the environment variable"
                " GOOGLE_APP_PROJECT_CLIENT_SECRET"
            )
            sys.exit(1)

        # Run through the OAuth flow and retrieve credentials
        flow = OAuth2WebServerFlow(client_id, client_secret, OAUTH_SCOPE,
                                   redirect_uri=REDIRECT_URI)

        # Indicate we need the user to grant us permissions and give the auth code or
        # not
        need_get_code = True

        if os.path.exists(CREDENTIALS_FILE) and use_credentials_cache:
            storage = Storage(CREDENTIALS_FILE)
            credentials = storage.get()
            _LOG.info("Use old credentials")
            need_get_code = False

        if need_get_code:
            authorize_url = flow.step1_get_authorize_url()
            print 'Go to the following link in your browser: ' + authorize_url
            code = raw_input('Enter verification code: ').strip()

            credentials = flow.step2_exchange(code)

            storage = Storage(CREDENTIALS_FILE)
            storage.put(credentials)

        # Create an httplib2.Http object and authorize it with our credentials
        http = httplib2.Http()
        http = credentials.authorize(http)

        self.drive_service = build('drive', 'v3', http=http)

    def list_action(self, args):
        self.auth(args.use_credentials_cache)
        files = self.list_path(args.path)
        for f in files:
            if args.detail:
                pprint.pprint(self.add_properties_to_file_metadata(f))
            else:
                print(f['name'])
        _LOG.info("Total of %s items", len(files))

    def list_path(self, path):
        if not path:
            path = '/'

        if path == '/':
            return self.list_path_by_parent_id('root')

        folder = self.get_folder_by_name(path)
        return self.list_path_by_parent_id(folder['id'])

    def get_folder_by_name(self, folder_name):
        response = self.drive_service.files().list(
            q="name contains '%s' and trashed = false" % folder_name,
            fields='nextPageToken, files(id, name, mimeType, parents)'
        ).execute()

        folders = [
            f for f in response['files']
            if f['mimeType'] == GDRIVE_MIMETYPE_FOLDER
        ]
        if len(folders) == 0:
            raise ResourceNotExists('Path "%s" not found' % folder_name)

        if len(folders) == 1:
            folder = folders[0]
        else:
            c = consoleutil.choose_item(
                "Choose the folder to list: ",
                [f['name'] for f in folders],
            )
            folder = folders[c]

        return folder

    def get_file_metadata(self, file_id):
        return self.drive_service.files().get(
            fileId=file_id, fields="id,name,mimeType,parents, webViewLink").execute()

    def list_path_by_parent_id(self, file_id):
        response = self.drive_service.files().list(
            q="'%s' in parents and trashed = false" % file_id,
            fields=(
                'nextPageToken, files(id, name, mimeType, parents, webViewLink)'
            ),
        ).execute()
        return response['files']

    def upload_action(self, args):
        self.auth(args.use_credentials_cache)
        self.upload_files(args.files, dest_dir=args.dest)

    def upload_files(self, filepaths, dest_dir=None):
        _LOG.info("Will upload files: %s" , ", ".join(filepaths))
        parent_id = None
        if dest_dir is not None:
            folder = self.get_folder_by_name(dest_dir)
            _LOG.info("Will upload file to folder: %s", folder['name'])
            parent_id = folder['id']

        for filepath in filepaths:
            if (filepath.startswith("http://") or
                filepath.startswith("https://")):
                basefn = os.path.basename(filepath)
                dest = os.path.join(tempfile.tempdir, basefn)
                _LOG.info("Download %s to %s", filepath, dest)
                webutil.download_file(filepath, dest)
                _LOG.info("Upload file: %s", dest)
                self.upload_file(dest, parent_id=parent_id)
            else:
                _LOG.info("Upload file: %s", filepath)
                self.upload_file(filepath, parent_id=parent_id)

    def upload_file(self, filepath, parent_id=None):
        if parent_id:
            parents = [parent_id]
        else:
            parents = []

        self.ensure_mimetype_exists(filepath)
        media = MediaFileUpload(
            filepath,
        )

        file_name = os.path.basename(filepath)
        response = self.drive_service.files().create(
            media_body=media,
            body={
                'name': file_name,
                'parents': parents,
            },
            fields='id, kind, mimeType, name, webViewLink, parents',
        ).execute()

        pprint.pprint(self.add_properties_to_file_metadata(response))

    def download_action(self, args):
        _LOG.error("Not implemented, yet")
        sys.exit(1)

    def search_action(self, args):
        self.search(args.query)

    def move_action(self, args):
        self.auth(args.use_credentials_cache)
        self.move_files(args.srcs, args.dest)

    def move_files(self, srcs, dest):
        srcs = [self.get_item(s) for s in srcs]
        dest = self.get_item(dest)

        for src in srcs:
            _LOG.info("Move %s to %s", src['name'], dest['name'])
            _LOG.debug("Src: %s", src)
            _LOG.debug("Dest: %s", dest)
            prev_parents = ",".join(src['parents'])
            self.drive_service.files().update(
                fileId=src['id'],
                addParents=dest['id'],
                removeParents=prev_parents,
                fields='id, kind, mimeType, name, webViewLink, parents',
            ).execute()

    def search_action(self, args):
        self.auth(args.use_credentials_cache)
        self.search(args.query)

    def search(self, query):
        if not query.startswith("q:"):
            print(self.get_file_metadata(query))
            return

        name = query[2:]

        response = self.drive_service.files().list(
            q="name contains '%s' and trashed = false" % name,
            fields='nextPageToken, files(id, name, mimeType, parents, webViewLink)'
        ).execute()

        for f in response['files']:
            print(f)

    def ensure_mimetype_exists(self, filepath):
        import mimetypes
        if mimetypes.guess_type(filepath)[0]:
            return

        _LOG.warn(
            "Cannot detect mimetype for %s, assume it's text/plain.",
            filepath,
        )

        ext = os.path.splitext(filepath)[1]
        # works most of the time, if not, too bad
        _LOG.info("Add type text/plain for extension %s", ext)
        mimetypes.add_type('text/plain', ext, strict=True)
        assert mimetypes.guess_type(filepath)

    def add_properties_to_file_metadata(self, file_metadata):
        parents = file_metadata.get('parents')
        if parents:
            parents_gdrive_links = [
                "https://drive.google.com/drive/folders/%s" % p
                for p in parents
            ]
            file_metadata['parents_gdrive_links'] = parents_gdrive_links

        return file_metadata

    def get_item(self, query):
        if query.startswith("q:"):
            return self.get_item_by_name(query[2:])
        else:
            return self.get_file_metadata(query)

    def get_item_by_name(self, name):
        response = self.drive_service.files().list(
            q="name contains '%s' and trashed = false" % name,
            fields='nextPageToken, files(id, name, mimeType, parents)'
        ).execute()

        items = [
            f for f in response['files']
        ]
        if len(items) == 0:
            raise ResourceNotExists('Item "%s" not found' % name)

        if len(items) == 1:
            item = items[0]
        else:
            c = consoleutil.choose_item(
                "Choose the folder to list: ",
                [f['name'] for f in items],
            )
            item = items[c]

        return item


class ResourceNotExists(Exception):
    pass


def main():
    app = App()
    args = parse_args(app, sys.argv[1:])

    loggingutil.setup_logger(_LOG, debug=args.debug)
    args.func(args)


def parse_args(app_obj, cmd_args):
    description = __doc__.split("Copyright 2014")[0].strip()
    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--no-credentials-cache",
        dest="use_credentials_cache",
        action="store_false",
        default=True,
        help="If specified, old credentials are not reused and you have to"
            " follow the instruction from this script to get the code every"
            " time you use this script.",
    )
    parser.add_argument(
        "--debug",
        action="store_true",
    )

    subparsers = parser.add_subparsers(title="Subcommands")

    #
    # list
    #

    parser_list = subparsers.add_parser(
        "list",
        help="List files in a folder",
        description="""
            List files in a folder, prompt for a choice if multiple folders
            found.
        """,
    )
    parser_list.add_argument(
        "path",
        nargs="?",
    )
    parser_list.add_argument(
        "--detail",
        action="store_true",
        help="List details for each file in the folder",
    )
    parser_list.set_defaults(func=app_obj.list_action)

    #
    # upload
    #

    parser_upload = subparsers.add_parser("upload")
    parser_upload.add_argument(
        "files",
        nargs="+",
        help="Local filepaths for files to be uploaded",
    )
    parser_upload.add_argument(
        "--dest",
        help="Destination folder the files uploaded to",
    )
    parser_upload.set_defaults(func=app_obj.upload_action)

    #
    # download
    #

    parser_download = subparsers.add_parser("download")
    parser_download.set_defaults(func=app_obj.download_action)

    #
    # move
    #

    parser_move = subparsers.add_parser(
        "move",
        help="Move files/folders to a folder"
    )
    parser_move.add_argument(
        "srcs",
        nargs="+",
        help="ID's for items to be moved. An ID can be an item name prefixed"
            " by q:"
    )
    parser_move.add_argument(
        "dest",
        help="ID for the items to move to. An ID can be an item name"
            " prefixed by q:"
    )
    parser_move.set_defaults(func=app_obj.move_action)

    #
    # search
    #

    parser_search = subparsers.add_parser(
        "search",
        help="Search files or folders"
    )
    parser_search.add_argument(
        "query",
        help="A query can be the ID of the file/folder, or the name prefixed"
            " by q:"
    )
    parser_search.set_defaults(func=app_obj.search_action)

    args = parser.parse_args(cmd_args)
    return args


if __name__ == '__main__':
    main()
